<?php

declare(strict_types = 1);

namespace Drupal\entity_visibility_preview\Plugin\EntityVisibilityPreviewCondition;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\entity_visibility_preview\EntityVisibilityPreviewConditionPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Allows to set a taxonomy term condition.
 *
 * @EntityVisibilityPreviewCondition(
 *   id = "taxonomy_condition",
 *   label = @Translation("Taxonomy condition"),
 *   dependencies = {"taxonomy"}
 * )
 */
class TaxonomyCondition extends EntityVisibilityPreviewConditionPluginBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Instance of EntityFieldManagerInterface manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  private $entityFieldManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * Taxonomy helper.
   *
   * @var \Drupal\entity_visibility_preview\Service\EntityVisibilityPreviewTaxonomyHelperInterface
   */
  protected $taxonomyHelper;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->entityFieldManager = $container->get('entity_field.manager');
    $instance->currentUser = $container->get('current_user');
    $instance->state = $container->get('state');
    $instance->entityTypeBundleInfo = $container->get('entity_type.bundle.info');
    $instance->taxonomyHelper = $container->get('entity_visibility_preview.taxonomy_helper');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm() : array {
    $form = [];

    $form['bundles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select one or more taxonomy vocabularies.'),
      '#options' => $this->getBundlesOptions(),
      '#default_value' => isset($this->configuration['bundles']) ? $this->configuration['bundles'] : [],
    ];

    $url = Url::fromRoute('entity_visibility_preview.taxonomy_condition_global_configuration_form');
    if ($url->renderAccess($url->toRenderArray())) {
      $form['bundles']['#description'] = $this->t('Configure the terms for the anonymous user on <a href=":url">the taxonomy condition settings page</a>.', [':url' => $url->toString()]);
    }

    $form['user_fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t("Select one or more user's taxonomy reference fields to compare with for the access."),
      '#options' => $this->getUserFieldsOptions(),
      '#default_value' => isset($this->configuration['user_fields']) ? $this->configuration['user_fields'] : [],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $values) : array {
    $default_value = [];
    if (isset($values['taxonomy_term_uuids'])) {
      $tids = $this->taxonomyHelper->getTidsFromUuids($values['taxonomy_term_uuids']);
      $default_value = $this->entityTypeManager->getStorage('taxonomy_term')
        ->loadMultiple($tids);
    }

    return [
      'taxonomy_term_tids' => [
        '#type' => 'entity_autocomplete',
        '#title' => $this->t('Categorization'),
        '#description' => $this->t('Enter at least one taxonomy term that the user must have to access to this content. If you enter multiple terms, only one is enough to access this content. Enter a comma-separated list.'),
        '#target_type' => 'taxonomy_term',
        '#selection_settings' => [
          'target_bundles' => $this->configuration['bundles'],
        ],
        '#tags' => TRUE,
        '#default_value' => $default_value,
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function formSubmit(array $values) : array {
    if (isset($values['taxonomy_term_tids'])) {
      $taxonomy_terms = [
        'taxonomy_term_uuids' => [],
      ];
      $termIds = [];
      foreach ($values['taxonomy_term_tids'] as $taxonomy) {
        $termIds[] = $taxonomy['target_id'];
      }

      $taxonomy_terms['taxonomy_term_uuids'] = $this->taxonomyHelper->getUuidsFromTids($termIds);
      return $taxonomy_terms;
    }

    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function accessCheck(array $values, FieldDefinitionInterface $field_definition) : AccessResultInterface {
    $cache_contexts = [
      'session',
    ];

    $access_result = AccessResult::neutral();
    if (!empty($values['taxonomy_term_uuids'])) {
      $access_result = AccessResult::forbidden();

      $actual_taxonomy = $this->getActualTaxonomy($field_definition);
      // Transform tids to uuids so we can compare with EVP field storage.
      $actual_taxonomy = $this->taxonomyHelper->getUuidsFromTids($actual_taxonomy);
      foreach ($values['taxonomy_term_uuids'] as $uuid) {
        if (in_array($uuid, $actual_taxonomy)) {
          $access_result = AccessResult::neutral();
          break;
        }
      }
    }

    return $access_result->addCacheContexts($cache_contexts);
  }

  /**
   * Get the actual user taxonomy, handling normal browsing and preview.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition of the field where the access check is called.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The account to evaluate taxonomies for.
   *   Default to current user.
   *
   * @return array
   *   The user taxonomy terms or the preview taxonomy terms.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getActualTaxonomy(FieldDefinitionInterface $field_definition, AccountInterface $account = NULL) : array {
    if (is_null($account)) {
      $account = $this->currentUser;
    }
    $user_taxonomy_terms = [];
    // Anonymous.
    if ($account->isAnonymous()) {
      $state_key_elements = [
        'entity_visibility_preview',
        'anonymous',
        'taxonomy_condition',
        $field_definition->getTargetEntityTypeId(),
        $field_definition->getTargetBundle(),
        $field_definition->getName(),
      ];

      $user_taxonomy_terms = $this->state->get(implode(':', $state_key_elements), []);
    }
    else {
      $preview_taxonomy = $this->sessionManager->getFromSession('entity_visibility_preview_taxonomy');

      // Preview case.
      if (!is_null($preview_taxonomy)) {
        $user_taxonomy_terms = $preview_taxonomy;
      }
      else {
        /** @var \Drupal\user\UserInterface $user */
        $user = $this->entityTypeManager->getStorage('user')->load($account->id());
        foreach ($this->configuration['user_fields'] as $user_field) {
          if ($user->hasField($user_field)) {
            /** @var \Drupal\taxonomy\TermInterface $user_field_taxonomy_terms */
            $user_field_taxonomy_terms = $user->get($user_field)->referencedEntities();
            foreach ($user_field_taxonomy_terms as $taxonomy_term) {
              $user_taxonomy_terms[] = $taxonomy_term->id();
            }
          }
        }
      }
    }

    return $user_taxonomy_terms;
  }

  /**
   * {@inheritdoc}
   */
  public function previewBuildForm(array &$form, FormStateInterface $form_state) : array {
    $default_value = [];
    $preview_taxonomy = $this->sessionManager->getFromSession('entity_visibility_preview_taxonomy');
    if (!is_null($preview_taxonomy)) {
      $default_value = $this->entityTypeManager->getStorage('taxonomy_term')
        ->loadMultiple($preview_taxonomy);
    }

    $bundle_options = $this->getBundlesOptions();
    $bundle_keys = array_keys($bundle_options);

    $form['preview_taxonomy'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('User taxonomy terms'),
      '#description' => $this->t('Enter the taxonomy terms simulated for a user. The taxonomy terms of the user account used for the preview will not be used. Enter a comma-separated list.'),
      '#target_type' => 'taxonomy_term',
      '#selection_settings' => [
        'target_bundles' => array_combine($bundle_keys, $bundle_keys),
      ],
      '#tags' => TRUE,
      '#default_value' => $default_value,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function previewSubmitForm(array &$form, FormStateInterface $form_state) : void {
    // Set the chosen display taxonomy terms in the session.
    $taxonomy_terms = [];
    $preview_taxonomy = $form_state->getValue(['preview_taxonomy']);
    if (!empty($preview_taxonomy)) {
      foreach ($preview_taxonomy as $taxonomy) {
        $taxonomy_terms[] = $taxonomy['target_id'];
      }
    }
    $this->sessionManager->setFromSession('entity_visibility_preview_taxonomy', $taxonomy_terms);
  }

  /**
   * {@inheritdoc}
   */
  public function previewResetForm() : void {
    $this->sessionManager->deleteFromSession('entity_visibility_preview_taxonomy');
  }

  /**
   * {@inheritdoc}
   */
  public function previewMessage(array &$items) : void {
    $preview_taxonomy_list = $this->sessionManager->getFromSession('entity_visibility_preview_taxonomy');
    if (!is_null($preview_taxonomy_list) && !empty($preview_taxonomy_list)) {
      $taxonomy_list = $this->entityTypeManager->getStorage('taxonomy_term')
        ->loadMultiple($preview_taxonomy_list);
      foreach ($taxonomy_list as $taxonomy) {
        $items[] = $this->t('User taxonomy: @taxonomy', [
          '@taxonomy' => $taxonomy->label(),
        ]);
      }
    }
  }

  /**
   * Get all the taxonomy term entity reference fields on user account.
   *
   * @return array
   *   The list of the taxonomy term entity reference fields on user account.
   */
  protected function getUserFieldsOptions() : array {
    $user_entity_reference_field = [];
    /** @var \Drupal\Core\Field\FieldDefinitionInterface[] $user_fields_definitions */
    $user_fields_definitions = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    foreach ($user_fields_definitions as $field_definition) {
      if ($field_definition->getType() == 'entity_reference') {
        if (
          isset($field_definition->getSettings()['handler']) &&
          $field_definition->getSettings()['handler'] == 'default:taxonomy_term'
        ) {
          $user_entity_reference_field[$field_definition->getName()] = $field_definition->getLabel();
        }
      }
    }
    return $user_entity_reference_field;
  }

  /**
   * The taxonomy terms bundle list to use in checkboxes options.
   *
   * @return array
   *   The taxonomy terms bundle list.
   */
  protected function getBundlesOptions() : array {
    $bundle_options = [];
    $bundles = $this->entityTypeBundleInfo->getBundleInfo('taxonomy_term');
    foreach ($bundles as $bundle_id => $bundle) {
      $bundle_options[$bundle_id] = $bundle['label'];
    }

    return $bundle_options;
  }

}
